unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, TAGraph, TASeries,
  TAMultiSeries, TAIntervalSources;

type

  { TForm1 }

  TForm1 = class(TForm)
    Chart1: TChart;
    Chart1LineSeries1: TLineSeries;
    Chart1OpenHighLowCloseSeries1: TOpenHighLowCloseSeries;
    DateTimeIntervalChartSource1: TDateTimeIntervalChartSource;
    procedure FormCreate(Sender: TObject);
  private
    procedure LoadData(AFileName: String);

  public

  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
begin
  LoadData('data.txt');
end;

procedure TForm1.LoadData(AFileName: String);
var
  List: TStringList;
  sa: TStringArray;
  i: Integer;
  D, prevD: TDate;       // date of this/previous record
  O, H, L, C: Double;    // OHLC of this day
  sumC: Double;          // sum of C values
  numC: Integer;         // count of C values in sum
  fs: TFormatSettings;
begin
  Chart1Lineseries1.Clear;
  Chart1OpenHighLowCloseSeries1.Clear;

  fs := DefaultFormatSettings;
  fs.DecimalSeparator := '.';
  fs.DateSeparator := '-';
  fs.ShortDateFormat := 'yyyy/mm/dd';

  prevD := 0;
  sumC := 0.0;
  numC := 0;

  List := TStringList.Create;
  try
    List.LoadFromFile(AFileName);
    for i := 1 to List.Count-1 do  // i := 1 --> skip header line
    begin
      sa := List[i].Split(',');
      if not (TryStrToDate(sa[0], D, fs) and
        TryStrToFloat(sa[1], O, fs) and TryStrToFloat(sa[2], H, fs) and
        TryStrToFloat(sa[3], L, fs) and TryStrToFloat(sa[4], C, fs)
      )
      then
        Continue;
      // Add OHLC values for every day to OHLC series
      Chart1OpenHighLowCloseSeries1.AddXOHLC(D, O, H, L, C);
      // On the first day of each week (i.e. when the last record is older than
      // 2 days) the average of the Close values of the preceeding week records
      // is calculated and added to the line series.
      if (D >= prevD + 2) and (numC > 0) then  // first day of week
      begin
        Chart1LineSeries1.AddXY(D, sumC / numC);
        sumC := 0.0;
        numC := 0;
      end;
      sumC := sumC + C;
      inc(numC);
      prevD := D;
    end;
  finally
    List.Free;
  end;
end;

end.

